@@ -1,21 +1,24 @@
 /*
- * Copyright (c) 1999-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -113,6 +116,7 @@ hfs_makelink(struct hfsmount *hfsmp, struct cnode *cp, struct cnode *dcp,
 	struct cat_desc to_desc;
 	int newlink = 0;
 	int retval;
+	cat_cookie_t cookie = {0};
 
 
 	/* We don't allow link nodes in our Private Meta Data folder! */
@@ -122,10 +126,15 @@ hfs_makelink(struct hfsmount *hfsmp, struct cnode *cp, struct cnode *dcp,
 	if (hfs_freeblks(hfsmp, 0) == 0)
 		return (ENOSPC);
 
+	/* Reserve some space in the Catalog file. */
+	if ((retval = cat_preflight(hfsmp, (2 * CAT_CREATE)+ CAT_RENAME, &cookie, p))) {
+		return (retval);
+	}
+
 	/* Lock catalog b-tree */
 	retval = hfs_metafilelocking(hfsmp, kHFSCatalogFileID, LK_EXCLUSIVE, p);
 	if (retval) {
-	    return retval;
+	   goto out2;
 	}
 
 	/*
@@ -216,7 +225,8 @@ hfs_makelink(struct hfsmount *hfsmp, struct cnode *cp, struct cnode *dcp,
 out:
 	/* Unlock catalog b-tree */
 	(void) hfs_metafilelocking(hfsmp, kHFSCatalogFileID, LK_RELEASE, p);
-
+out2:
+	cat_postflight(hfsmp, &cookie, p);
 	return (retval);
 }
 
@@ -232,6 +242,7 @@ out:
      IN struct componentname *cnp;
 
      */
+__private_extern__
 int
 hfs_link(ap)
 	struct vop_link_args /* {
@@ -264,7 +275,7 @@ hfs_link(ap)
 	if (VTOVCB(tdvp)->vcbSigWord != kHFSPlusSigWord)
 		return err_link(ap);	/* hfs disks don't support hard links */
 	
-	if (hfsmp->hfs_private_metadata_dir == 0)
+	if (hfsmp->hfs_privdir_desc.cd_cnid == 0)
 		return err_link(ap);	/* no private metadata dir, no links possible */
 
 	if (tdvp != vp && (error = vn_lock(vp, LK_EXCLUSIVE, p))) {
@@ -294,7 +305,9 @@ hfs_link(ap)
 	if (hfsmp->jnl) {
 	    if (journal_start_transaction(hfsmp->jnl) != 0) {
 			hfs_global_shared_lock_release(hfsmp);
-			return EINVAL;
+			VOP_ABORTOP(tdvp, cnp);
+			error = EINVAL;  /* cannot link to a special file */
+			goto out1;
 	    }
 	}
 
@@ -324,13 +337,23 @@ hfs_link(ap)
 	// XXXdbg - need to do this here as well because cp could have changed
 	error = VOP_UPDATE(vp, &tv, &tv, 1);
 
-	FREE_ZONE(cnp->cn_pnbuf, cnp->cn_pnlen, M_NAMEI);
 
 	if (hfsmp->jnl) {
 	    journal_end_transaction(hfsmp->jnl);
 	}
 	hfs_global_shared_lock_release(hfsmp);
 
+	/* free the pathname buffer */
+	{
+		char *tmp = cnp->cn_pnbuf;
+		cnp->cn_pnbuf = NULL;
+		cnp->cn_flags &= ~HASBUF;
+		FREE_ZONE(tmp, cnp->cn_pnlen, M_NAMEI);
+	}
+	
+	HFS_KNOTE(vp, NOTE_LINK);
+	HFS_KNOTE(tdvp, NOTE_WRITE);
+
 out1:
 	if (tdvp != vp)
 		VOP_UNLOCK(vp, 0, p);
