@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -65,6 +65,7 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/sysctl.h>
+#include <sys/kern_audit.h>
 
 #include <mach/mach_types.h>
 #include <mach/vm_inherit.h>
@@ -120,7 +121,7 @@ struct shmmap_state {
 static void shm_deallocate_segment __P((struct shmid_ds *));
 static int shm_find_segment_by_key __P((key_t));
 static struct shmid_ds *shm_find_segment_by_shmid __P((int));
-static int shm_delete_mapping __P((struct proc *, struct shmmap_state *));
+static int shm_delete_mapping __P((struct proc *, struct shmmap_state *, int));
 
 #ifdef __APPLE_API_PRIVATE
 struct  shminfo shminfo = {
@@ -183,9 +184,10 @@ shm_deallocate_segment(shmseg)
 }
 
 static int
-shm_delete_mapping(p, shmmap_s)
+shm_delete_mapping(p, shmmap_s, deallocate)
 	struct proc *p;
 	struct shmmap_state *shmmap_s;
+	int deallocate;
 {
 	struct shmid_ds *shmseg;
 	int segnum, result;
@@ -194,9 +196,11 @@ shm_delete_mapping(p, shmmap_s)
 	segnum = IPCID_TO_IX(shmmap_s->shmid);
 	shmseg = &shmsegs[segnum];
 	size = round_page_32(shmseg->shm_segsz);
+	if (deallocate) {
 	result = vm_deallocate(current_map(), shmmap_s->va, size);
 	if (result != KERN_SUCCESS)
 		return EINVAL;
+	}
 	shmmap_s->shmid = -1;
 	shmseg->shm_dtime = time_second;
 	if ((--shmseg->shm_nattch <= 0) &&
@@ -220,6 +224,7 @@ shmdt(p, uap, retval)
 	struct shmmap_state *shmmap_s;
 	int i;
 
+	AUDIT_ARG(svipc_addr, uap->shmaddr);
 	if (!shm_inited)
 		return(EINVAL);
 	shmmap_s = (struct shmmap_state *)p->vm_shm;
@@ -231,7 +236,7 @@ shmdt(p, uap, retval)
 			break;
 	if (i == shminfo.shmseg)
 		return EINVAL;
-	return shm_delete_mapping(p, shmmap_s);
+	return shm_delete_mapping(p, shmmap_s, 1);
 }
 
 #ifndef _SYS_SYSPROTO_H_
@@ -258,6 +263,8 @@ shmat(p, uap, retval)
 	vm_size_t size;
 	kern_return_t rv;
 
+	AUDIT_ARG(svipc_id, uap->shmid);
+	AUDIT_ARG(svipc_addr, uap->shmaddr);
 	if (!shm_inited)
 		return(EINVAL);
 	shmmap_s = (struct shmmap_state *)p->vm_shm;
@@ -271,6 +278,8 @@ shmat(p, uap, retval)
 	shmseg = shm_find_segment_by_shmid(uap->shmid);
 	if (shmseg == NULL)
 		return EINVAL;
+
+	AUDIT_ARG(svipc_perm, &shmseg->shm_perm);
 	error = ipcperm(cred, &shmseg->shm_perm,
 	    (uap->shmflg & SHM_RDONLY) ? IPC_R : IPC_R|IPC_W);
 	if (error)
@@ -413,11 +422,18 @@ shmctl(p, uap, 	retval)
 	struct shmid_ds inbuf;
 	struct shmid_ds *shmseg;
 
+	AUDIT_ARG(svipc_cmd, uap->cmd);
+	AUDIT_ARG(svipc_id, uap->shmid);
 	if (!shm_inited)
 		return(EINVAL);
 	shmseg = shm_find_segment_by_shmid(uap->shmid);
 	if (shmseg == NULL)
 		return EINVAL;
+	/* XXAUDIT: This is the perms BEFORE any change by this call. This 
+	 * may not be what is desired.
+	 */
+	AUDIT_ARG(svipc_perm, &shmseg->shm_perm);
+
 	switch (uap->cmd) {
 	case IPC_STAT:
 		error = ipcperm(cred, &shmseg->shm_perm, IPC_R);
@@ -573,6 +589,7 @@ shmget_allocate_segment(p, uap, mode, retval)
 	shmseg->shm_ctime = time_second;
 	shm_committed += btoc(size);
 	shm_nused++;
+	AUDIT_ARG(svipc_perm, &shmseg->shm_perm);
 	if (shmseg->shm_perm.mode & SHMSEG_WANTED) {
 		/*
 		 * Somebody else wanted this key while we were asleep.  Wake
@@ -582,6 +599,7 @@ shmget_allocate_segment(p, uap, mode, retval)
 		wakeup((caddr_t)shmseg);
 	}
 	*retval = shmid;
+	AUDIT_ARG(svipc_id, shmid);
 	return 0;
 out: 
 	switch (kret) {
@@ -604,6 +622,7 @@ shmget(p, uap, retval)
 {
 	int segnum, mode, error;
 
+	/* Auditing is actually done in shmget_allocate_segment() */
 	if (!shm_inited)
 		return(EINVAL);
 
@@ -676,7 +695,28 @@ shmexit(p)
 	shmmap_s = (struct shmmap_state *)p->vm_shm;
 	for (i = 0; i < shminfo.shmseg; i++, shmmap_s++)
 		if (shmmap_s->shmid != -1)
-			shm_delete_mapping(p, shmmap_s);
+			shm_delete_mapping(p, shmmap_s, 1);
+	FREE((caddr_t)p->vm_shm, M_SHM);
+	p->vm_shm = NULL;
+}
+
+/*
+ * shmexec() is like shmexit(), only it doesn't delete the mappings,
+ * since the old address space has already been destroyed and the new
+ * one instantiated.  Instead, it just does the housekeeping work we
+ * need to do to keep the System V shared memory subsystem sane.
+ */
+__private_extern__ void
+shmexec(p)
+	struct proc *p;
+{
+	struct shmmap_state *shmmap_s;
+	int i;
+
+	shmmap_s = (struct shmmap_state *)p->vm_shm;
+	for (i = 0; i < shminfo.shmseg; i++, shmmap_s++)
+		if (shmmap_s->shmid != -1)
+			shm_delete_mapping(p, shmmap_s, 0);
 	FREE((caddr_t)p->vm_shm, M_SHM);
 	p->vm_shm = NULL;
 }
@@ -732,7 +772,7 @@ sysctl_shminfo SYSCTL_HANDLER_ARGS
 			(shminfo.shmmni != -1) &&
 			(shminfo.shmseg != -1) &&
 			(shminfo.shmall != -1)) {
-				shminit();
+				shminit(NULL);
 		}
 	}
 	return(0);
