@@ -3,19 +3,22 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -50,7 +53,7 @@
 
 static void atp_trans_complete();
 void atp_x_done();
-void atp_x_done_funnel();
+void atp_x_done_funnel(void *);
 extern void atp_req_timeout();
 
 /*
@@ -58,9 +61,9 @@ extern void atp_req_timeout();
  *	Version 1.7 of atp_read.c on 89/02/09 17:53:16
  */
 
-void atp_treq_event(gref)
-register gref_t *gref;
+void atp_treq_event(void *arg)
 {
+	register gref_t *gref = (gref_t *)arg;
 	register gbuf_t *m;
 	register struct atp_state *atp;
 	boolean_t 	funnel_state;
@@ -456,10 +459,10 @@ gbuf_t   *m;
 
 void 
 atp_x_done_funnel(trp)
-register struct atp_trans *trp;
+void *trp;
 {
 	thread_funnel_set(network_flock, TRUE);
-	atp_x_done(trp);
+	atp_x_done((struct atp_trans *)trp);
 	(void) thread_funnel_set(network_flock, FALSE);
 
 }
